#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "misc.h"

/* doconnect()
 * perform actual connection to a host
 * uses opt_p for port
 * adjusts socket options
 * returns socket descriptor
 * exits 1 on error (should always return a valid sd)
 */

int doconnect (host)
	char *host;
{
	struct protoent *pr;
	struct sockaddr_in saddr;
	int sd, rval;
	lx_s ns = {0};

	/* get protocol info */
	if (!(pr = getprotobyname("tcp"))) {
		outlog (gderr, "getprotobyname: ", strerror(errno), 0,0,0);
		_exit(1);
	}

	/* open a tcp socket into sd */
	if ((sd = socket (AF_INET, SOCK_STREAM, pr->p_proto)) < 0) {
		outlog (gderr, "socket: ", strerror(errno), 0,0,0);
		_exit(1);
	}

	/* modify receive buffer */
	if (sockbuf.recv) {
		if (lx_straddulong (&ns, (unsigned long)sockbuf.recv, 10)
				|| lx_check0(&ns)) die_nomem();
		debuglog (2, gderr, "setting receive buffer to ", ns.s, 0,0,0);
		rval = setsockopt (sd, SOL_SOCKET, SO_RCVBUF,
			&sockbuf.recv, sizeof (sockbuf.recv));
		if (rval) die_setsockopt(sd, "SO_RCVBUF");
		lx_free (&ns);
	}

	/* modify send buffer */
	if (sockbuf.send) {
		if (lx_straddulong (&ns, (unsigned long)sockbuf.send, 10)
				|| lx_check0(&ns)) die_nomem();
		rval = setsockopt (sd, SOL_SOCKET, SO_SNDBUF,
			&sockbuf.send, sizeof (sockbuf.send));
		if (rval) die_setsockopt(sd, "SO_SNDBUF");
		lx_free (&ns);
	}


	/* clear and populate saddr */
	memset (&saddr, 0, sizeof(saddr));

	if (!inet_aton(host, (struct in_addr *)&saddr.sin_addr)) {
		outlog (gderr, host, " is not a valid address", 0,0,0);
		close (sd);
		_exit(1);
	}

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(opt_p.v_opt.opt_ulong);

	if (connect (sd, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) {
		outlog (gderr, "connect: ", strerror(errno), 0,0,0);
		close (sd);
		_exit(1);
	}

	return sd;
}
